-------------------------------------------------------------------------------
-- Title      : Application Acceleration FPGA Top Level File 
-- Project    : Application Acceleration FPGA
-------------------------------------------------------------------------------
-- File       : $RCSfile: top.vhd,v $
-- Author     : Cray Canada
-- Company    : Cray Canada Inc
-- Created    : 2004-12-22
-- Last update: 2005-01-28
-------------------------------------------------------------------------------
-- Description:
-------------------------------------------------------------------------------
-- Copyright (c) 2004 OctigaBay Systems Corporation
-------------------------------------------------------------------------------
--
-- Adapted for Impulse C
--
-- Revision 1.1  2005/02/01 00:10:57 
-- Moved up to this level from top subdirectory.
--
-- Revision 1.1  2004/12/22 23:54:40 
-- Initial checkin.
--
-------------------------------------------------------------------------------

library ieee, unisim;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use unisim.vcomponents.all;

-- pragma synthesis_off
library rt, qdr;
use rt.all;
use qdr.all;
-- pragma synthesis_on

entity top is
  port (
    glbl_reset_n  : in    std_logic;                      -- Reset
    -- RT I/F
    rt_clk_p      : in    std_logic;                      -- RT clock
    rt_clk_n      : in    std_logic;                      -- RT clock
    rkta_clk_p    : in    std_logic;                      -- RocketIO clock A
    rkta_clk_n    : in    std_logic;                      -- RocketIO clock A
    rktb_clk_p    : in    std_logic;                      -- RocketIO clock B
    rktb_clk_n    : in    std_logic;                      -- RocketIO clock B
    prog_clk_p    : in    std_logic;                      -- Programmable clock
    prog_clk_n    : in    std_logic;                      -- Programmable clock
    rt_rx_clk_p   : in    std_logic;
    rt_rx_clk_n   : in    std_logic;
    rt_rx_ctl_p   : in    std_logic;
    rt_rx_ctl_n   : in    std_logic;
    rt_rx_cad_p   : in    std_logic_vector(15 downto 0);
    rt_rx_cad_n   : in    std_logic_vector(15 downto 0);
    rt_rx_par_p   : in    std_logic_vector(1 downto 0);
    rt_rx_par_n   : in    std_logic_vector(1 downto 0);
    rt_tx_clk_p   : out   std_logic;
    rt_tx_clk_n   : out   std_logic;
    rt_tx_ctl_p   : out   std_logic;
    rt_tx_ctl_n   : out   std_logic;
    rt_tx_cad_p   : out   std_logic_vector(15 downto 0);
    rt_tx_cad_n   : out   std_logic_vector(15 downto 0);
    rt_tx_par_p   : out   std_logic_vector(1 downto 0);
    rt_tx_par_n   : out   std_logic_vector(1 downto 0);
    -- Host Processor I/F
    host_cs_n     : in    std_logic;
    host_as       : in    std_logic;
    host_rw       : in    std_logic;
    host_par      : inout std_logic;
    host_ad       : inout std_logic_vector(7 downto 0);
    host_int_n    : out   std_logic;
    host_grn_led  : out   std_logic;
    -- Frequency Generator I/F
    s_clock       : out   std_logic;
    s_data        : out   std_logic;
    s_load        : out   std_logic;
    np_load       : out   std_logic;
    prog_en_n     : out   std_logic;
    -- Programmable Clock Feedback
    k_copy_out    : out   std_logic;
    k_copy_in     : in    std_logic;
    -- QDR RAM I/F
    qdr_rfb_n_1   : in    std_logic;                      -- RAM read feedback
    qdr_dr_1      : in    std_logic_vector(35 downto 0);  -- RAM read data
    qdr_dw_1      : out   std_logic_vector(35 downto 0);  -- RAM write data
    qdr_sa_1      : out   std_logic_vector(19 downto 0);  -- RAM address
    qdr_r_n_1     : out   std_logic;                      -- RAM read strobe
    qdr_w_n_1     : out   std_logic;                      -- RAM write strobe
    qdr_bw_n_1    : out   std_logic_vector(3 downto 0);   -- RAM byte write 
    qdr_k_1       : out   std_logic;                      -- RAM K clock
    qdr_k_n_1     : out   std_logic;                      -- RAM K clock invert
    qdr_cq_1      : in    std_logic;                      -- RAM C clock
    qdr_rext_n_1  : out   std_logic;                      -- RAM read external
    qdr_doff_n_1  : out   std_logic;                      -- RAM DLL Off
    qdr_rfb_n_2   : in    std_logic;                      -- RAM read feedback
    qdr_dr_2      : in    std_logic_vector(35 downto 0);  -- RAM read data
    qdr_dw_2      : out   std_logic_vector(35 downto 0);  -- RAM write data
    qdr_sa_2      : out   std_logic_vector(19 downto 0);  -- RAM address
    qdr_r_n_2     : out   std_logic;                      -- RAM read strobe
    qdr_w_n_2     : out   std_logic;                      -- RAM write strobe
    qdr_bw_n_2    : out   std_logic_vector(3 downto 0);   -- RAM byte write 
    qdr_k_2       : out   std_logic;                      -- RAM K clock
    qdr_k_n_2     : out   std_logic;                      -- RAM K clock invert
    qdr_cq_2      : in    std_logic;                      -- RAM C clock
    qdr_rext_n_2  : out   std_logic;                      -- RAM read external
    qdr_doff_n_2  : out   std_logic;                      -- RAM DLL Off
    qdr_rfb_n_3   : in    std_logic;                      -- RAM read feedback
    qdr_dr_3      : in    std_logic_vector(35 downto 0);  -- RAM read data
    qdr_dw_3      : out   std_logic_vector(35 downto 0);  -- RAM write data
    qdr_sa_3      : out   std_logic_vector(19 downto 0);  -- RAM address
    qdr_r_n_3     : out   std_logic;                      -- RAM read strobe
    qdr_w_n_3     : out   std_logic;                      -- RAM write strobe
    qdr_bw_n_3    : out   std_logic_vector(3 downto 0);   -- RAM byte write 
    qdr_k_3       : out   std_logic;                      -- RAM K clock
    qdr_k_n_3     : out   std_logic;                      -- RAM K clock invert
    qdr_cq_3      : in    std_logic;                      -- RAM C clock
    qdr_rext_n_3  : out   std_logic;                      -- RAM read external
    qdr_doff_n_3  : out   std_logic;                      -- RAM DLL Off
    qdr_rfb_n_4   : in    std_logic;                      -- RAM read feedback
    qdr_dr_4      : in    std_logic_vector(35 downto 0);  -- RAM read data
    qdr_dw_4      : out   std_logic_vector(35 downto 0);  -- RAM write data
    qdr_sa_4      : out   std_logic_vector(19 downto 0);  -- RAM address
    qdr_r_n_4     : out   std_logic;                      -- RAM read strobe
    qdr_w_n_4     : out   std_logic;                      -- RAM write strobe
    qdr_bw_n_4    : out   std_logic_vector(3 downto 0);   -- RAM byte write 
    qdr_k_4       : out   std_logic;                      -- RAM K clock
    qdr_k_n_4     : out   std_logic;                      -- RAM K clock invert
    qdr_cq_4      : in    std_logic;                      -- RAM C clock
    qdr_rext_n_4  : out   std_logic;                      -- RAM read external
    qdr_doff_n_4  : out   std_logic;                      -- RAM DLL Off
    -- Debug
    prog_clk0_out : out   std_logic;
    gnd_out       : out   std_logic;
    err           : out   std_logic;
    err_code      : out   std_logic_vector(2 downto 0);
    data_out      : out   std_logic_vector(1 downto 0)
    );
end entity top;

architecture rtl of top is

  -----------------------------------------------------------------------------
  -- Declare components.
  -----------------------------------------------------------------------------
  attribute box_type : string;

  component ibuf_hstl_i_18
    port (i : in  std_logic;
          o : out std_logic);
  end component;
  attribute box_type of ibuf_hstl_i_18 : component is "black_box";

  component obuf_hstl_i_18
    port (i : in  std_logic;
          o : out std_logic);
  end component;
  attribute box_type of obuf_hstl_i_18 : component is "black_box";

  component fddrrse
    port(q  : out std_logic;
         d0 : in  std_logic;
         d1 : in  std_logic;
         c0 : in  std_logic;
         c1 : in  std_logic;
         ce : in  std_logic;
         r  : in  std_logic;
         s  : in  std_logic);
  end component;
  attribute box_type of fddrrse : component is "black_box";

  component ibuf_lvcmos2 is
    port (
      i : in  std_logic;
      o : out std_logic);
  end component ibuf_lvcmos2;
  attribute box_type of ibuf_lvcmos2 : component is "black_box";

  component obuf_lvcmos2 is
    port (
      i : in  std_logic;
      o : out std_logic);
  end component obuf_lvcmos2;
  attribute box_type of obuf_lvcmos2 : component is "black_box";

  component iobuf_lvcmos2
    port (i  : in    std_logic;
          t  : in    std_logic;
          o  : out   std_logic;
          io : inout std_logic);
  end component;
  attribute box_type of iobuf_lvcmos2 : component is "black_box";

  component ibufgds_lvpecl_25 is
    port (
      i  : in  std_logic;
      ib : in  std_logic;
      o  : out std_logic);
  end component ibufgds_lvpecl_25;
  attribute box_type of ibufgds_lvpecl_25 : component is "black_box";

  component ibufg_hstl_i_18
    port (i : in  std_logic;
          o : out std_logic);
  end component;
  attribute box_type of ibufg_hstl_i_18 : component is "black_box";

  component bufg is
    port (
      i : in  std_logic;
      o : out std_logic);
  end component bufg;
  attribute box_type of bufg : component is "black_box";

  component dcm
    generic (
      clk_feedback          : string  := "1X";
      clkdv_divide          : real    := 2.0;
      clkfx_divide          : integer := 1;
      clkfx_multiply        : integer := 4;
      clkin_divide_by_2     : boolean := false;   -- (true, false)
      clkin_period          : real    := 0.0;
      clkout_phase_shift    : string  := "NONE";
      deskew_adjust         : string  := "SYSTEM_SYNCHRONOUS";
      dfs_frequency_mode    : string  := "LOW";
      dll_frequency_mode    : string  := "LOW";
      dss_mode              : string  := "NONE";
      duty_cycle_correction : boolean := true;    -- (true, false)
      phase_shift           : integer := 0;
      startup_wait          : boolean := false);  -- (true, false)
    port (clk0     : out std_ulogic;
          clk180   : out std_ulogic;
          clk270   : out std_ulogic;
          clk2x    : out std_ulogic;
          clk2x180 : out std_ulogic;
          clk90    : out std_ulogic;
          clkdv    : out std_ulogic;
          clkfx    : out std_ulogic;
          clkfx180 : out std_ulogic;
          locked   : out std_ulogic;
          psdone   : out std_ulogic;
          status   : out std_logic_vector (7 downto 0);
          clkfb    : in  std_ulogic;
          clkin    : in  std_ulogic;
          dssen    : in  std_ulogic;
          psclk    : in  std_ulogic;
          psen     : in  std_ulogic;
          psincdec : in  std_ulogic;
          rst      : in  std_ulogic);
  end component;
  attribute box_type of dcm : component is "black_box";

  component srl16
    generic (
      init : bit_vector := x"0001");
    port (q   : out std_ulogic;
          a0  : in  std_ulogic;
          a1  : in  std_ulogic;
          a2  : in  std_ulogic;
          a3  : in  std_ulogic;
          clk : in  std_ulogic;
          d   : in  std_ulogic);
  end component;
  attribute box_type of srl16 : component is "black_box";

  component clock_macro is
    generic (
      dcm_location : string);
    port (
      clock1_in  : in  std_logic;
      clock2_in  : in  std_logic;
      clock1_out : out std_logic;
      clock2_out : out std_logic);
  end component clock_macro;
  
  component prog_clock_gen is
    port (
      reset         : in  std_logic;
      prog_clk_in   : in  std_logic;
      k_copy_fb     : in  std_logic;
      prog_clk0     : out std_logic;
      prog_clk0_dcm : out std_logic;
      prog_clk90    : out std_logic;
      prog_clk180   : out std_logic;
      prog_dcm_lock : out std_logic); 
  end component prog_clock_gen;

  component rt_core is
    port (
      reset_n            : in    std_logic;
      user_clk           : in    std_logic;
      user_enable        : in    std_logic;
      ureq_addr          : in    std_logic_vector(39 downto 3);
      ureq_size          : in    std_logic_vector(2 downto 0);
      ureq_mask          : in    std_logic_vector(7 downto 0);
      ureq_rw_n          : in    std_logic;
      ureq_data          : in    std_logic_vector(63 downto 0);
      ureq_ts            : in    std_logic;
      ureq_byte_req      : in    std_logic;
      ureq_srctag        : out   std_logic_vector(4 downto 0);
      ureq_full          : out   std_logic;
      ureq_notag         : out   std_logic;
      fresp_enable       : in    std_logic;
      fresp_valid        : out   std_logic;
      fresp_ts           : out   std_logic;
      fresp_size         : out   std_logic_vector(2 downto 0);
      fresp_srctag       : out   std_logic_vector(4 downto 0);
      fresp_data         : out   std_logic_vector(63 downto 0);
      uresp_ts           : in    std_logic;
      uresp_size         : in    std_logic_vector(3 downto 0);
      uresp_srctag       : in    std_logic_vector(4 downto 0);
      uresp_data         : in    std_logic_vector(63 downto 0);
      uresp_full         : out   std_logic;
      freq_enable        : in    std_logic;
      freq_addr          : out   std_logic_vector(39 downto 3);
      freq_size          : out   std_logic_vector(3 downto 0);
      freq_srctag        : out   std_logic_vector(4 downto 0);
      freq_mask          : out   std_logic_vector(7 downto 0);
      freq_rw_n          : out   std_logic;
      freq_ts            : out   std_logic;
      freq_valid         : out   std_logic;
      freq_data          : out   std_logic_vector(63 downto 0);
      rt_rx_clk_p        : in    std_logic;
      rt_rx_clk_n        : in    std_logic;
      rt_rx_ctl_p        : in    std_logic;
      rt_rx_ctl_n        : in    std_logic;
      rt_rx_cad_p        : in    std_logic_vector(15 downto 0);
      rt_rx_cad_n        : in    std_logic_vector(15 downto 0);
      rt_rx_par_p        : in    std_logic_vector(1 downto 0);
      rt_rx_par_n        : in    std_logic_vector(1 downto 0);
      rt_clk_p           : in    std_logic;
      rt_clk_n           : in    std_logic;
      rt_tx_clk_p        : out   std_logic;
      rt_tx_clk_n        : out   std_logic;
      rt_tx_ctl_p        : out   std_logic;
      rt_tx_ctl_n        : out   std_logic;
      rt_tx_cad_p        : out   std_logic_vector(15 downto 0);
      rt_tx_cad_n        : out   std_logic_vector(15 downto 0);
      rt_tx_par_p        : out   std_logic_vector(1 downto 0);
      rt_tx_par_n        : out   std_logic_vector(1 downto 0);
      clk_200m           : out   std_logic;
      rt_ready           : out   std_logic;
      host_cs_n          : in    std_logic;
      host_as            : in    std_logic;
      host_rw            : in    std_logic;
      host_par           : inout std_logic;
      host_ad            : inout std_logic_vector(7 downto 0);
      host_int_n         : out   std_logic;
      host_grn_led       : out   std_logic;
      ram_doff_n         : out   std_logic;
      s_clock            : out   std_logic;
      s_data             : out   std_logic;
      s_load             : out   std_logic;
      np_load            : out   std_logic;
      prog_en_n          : out   std_logic;
      prog_dcm_reset     : out   std_logic;
      user_reset_n       : out   std_logic);
  end component rt_core;
  attribute box_type of rt_core : component is "black_box";

  component qdr2_core is
    port (
      sim_mode     : in  std_logic;
      reset        : in  std_logic;
      qdr_clk0     : in  std_logic;
      qdr_clk90    : in  std_logic;
      locked_dcm   : in  std_logic;
      ram_rdy      : out std_logic;
      dw_1         : in  std_logic_vector(71 downto 0);
      dr_1         : out std_logic_vector(71 downto 0);
      ar_1         : in  std_logic_vector(19 downto 0);
      aw_1         : in  std_logic_vector(19 downto 0);
      r_n_1        : in  std_logic;
      w_n_1        : in  std_logic;
      bw_n_1       : in  std_logic_vector(7 downto 0);
      doff_n_1     : in  std_logic;
      dw_2         : in  std_logic_vector(71 downto 0);
      dr_2         : out std_logic_vector(71 downto 0);
      ar_2         : in  std_logic_vector(19 downto 0);
      aw_2         : in  std_logic_vector(19 downto 0);
      r_n_2        : in  std_logic;
      w_n_2        : in  std_logic;
      bw_n_2       : in  std_logic_vector(7 downto 0);
      doff_n_2     : in  std_logic;
      dw_3         : in  std_logic_vector(71 downto 0);
      dr_3         : out std_logic_vector(71 downto 0);
      ar_3         : in  std_logic_vector(19 downto 0);
      aw_3         : in  std_logic_vector(19 downto 0);
      r_n_3        : in  std_logic;
      w_n_3        : in  std_logic;
      bw_n_3       : in  std_logic_vector(7 downto 0);
      doff_n_3     : in  std_logic;
      dw_4         : in  std_logic_vector(71 downto 0);
      dr_4         : out std_logic_vector(71 downto 0);
      ar_4         : in  std_logic_vector(19 downto 0);
      aw_4         : in  std_logic_vector(19 downto 0);
      r_n_4        : in  std_logic;
      w_n_4        : in  std_logic;
      bw_n_4       : in  std_logic_vector(7 downto 0);
      doff_n_4     : in  std_logic;
      qdr_rfb_n_1  : in  std_logic;
      qdr_dr_1     : in  std_logic_vector(35 downto 0);
      qdr_dw_1     : out std_logic_vector(35 downto 0);
      qdr_sa_1     : out std_logic_vector(19 downto 0);
      qdr_r_n_1    : out std_logic;
      qdr_w_n_1    : out std_logic;
      qdr_bw_n_1   : out std_logic_vector(3 downto 0);
      qdr_k_1      : out std_logic;
      qdr_k_n_1    : out std_logic;
      qdr_cq_1     : in  std_logic;
      qdr_rext_n_1 : out std_logic;
      qdr_doff_n_1 : out std_logic;
      qdr_rfb_n_2  : in  std_logic;
      qdr_dr_2     : in  std_logic_vector(35 downto 0);
      qdr_dw_2     : out std_logic_vector(35 downto 0);
      qdr_sa_2     : out std_logic_vector(19 downto 0);
      qdr_r_n_2    : out std_logic;
      qdr_w_n_2    : out std_logic;
      qdr_bw_n_2   : out std_logic_vector(3 downto 0);
      qdr_k_2      : out std_logic;
      qdr_k_n_2    : out std_logic;
      qdr_cq_2     : in  std_logic;
      qdr_rext_n_2 : out std_logic;
      qdr_doff_n_2 : out std_logic;
      qdr_rfb_n_3  : in  std_logic;
      qdr_dr_3     : in  std_logic_vector(35 downto 0);
      qdr_dw_3     : out std_logic_vector(35 downto 0);
      qdr_sa_3     : out std_logic_vector(19 downto 0);
      qdr_r_n_3    : out std_logic;
      qdr_w_n_3    : out std_logic;
      qdr_bw_n_3   : out std_logic_vector(3 downto 0);
      qdr_k_3      : out std_logic;
      qdr_k_n_3    : out std_logic;
      qdr_cq_3     : in  std_logic;
      qdr_rext_n_3 : out std_logic;
      qdr_doff_n_3 : out std_logic;
      qdr_rfb_n_4  : in  std_logic;
      qdr_dr_4     : in  std_logic_vector(35 downto 0);
      qdr_dw_4     : out std_logic_vector(35 downto 0);
      qdr_sa_4     : out std_logic_vector(19 downto 0);
      qdr_r_n_4    : out std_logic;
      qdr_w_n_4    : out std_logic;
      qdr_bw_n_4   : out std_logic_vector(3 downto 0);
      qdr_k_4      : out std_logic;
      qdr_k_n_4    : out std_logic;
      qdr_cq_4     : in  std_logic;
      qdr_rext_n_4 : out std_logic;
      qdr_doff_n_4 : out std_logic); 
  end component qdr2_core;
  attribute box_type of qdr2_core : component is "black_box";

  component user_app is
    port (
      reset_n       : in  std_logic;
      user_clk      : in  std_logic;
      user_enable   : in  std_logic;
      rt_ready      : in  std_logic;
      qdr_ready     : in  std_logic;
      dr_1          : in  std_logic_vector(71 downto 0);
      ar_1          : out std_logic_vector(19 downto 0);
      aw_1          : out std_logic_vector(19 downto 0);
      r_n_1         : out std_logic;
      w_n_1         : out std_logic;
      bw_n_1        : out std_logic_vector(7 downto 0);
      dw_1          : out std_logic_vector(71 downto 0);
      dr_2          : in  std_logic_vector(71 downto 0);
      ar_2          : out std_logic_vector(19 downto 0);
      aw_2          : out std_logic_vector(19 downto 0);
      r_n_2         : out std_logic;
      w_n_2         : out std_logic;
      bw_n_2        : out std_logic_vector(7 downto 0);
      dw_2          : out std_logic_vector(71 downto 0);
      dr_3          : in  std_logic_vector(71 downto 0);
      ar_3          : out std_logic_vector(19 downto 0);
      aw_3          : out std_logic_vector(19 downto 0);
      r_n_3         : out std_logic;
      w_n_3         : out std_logic;
      bw_n_3        : out std_logic_vector(7 downto 0);
      dw_3          : out std_logic_vector(71 downto 0);
      dr_4          : in  std_logic_vector(71 downto 0);
      ar_4          : out std_logic_vector(19 downto 0);
      aw_4          : out std_logic_vector(19 downto 0);
      r_n_4         : out std_logic;
      w_n_4         : out std_logic;
      bw_n_4        : out std_logic_vector(7 downto 0);
      dw_4          : out std_logic_vector(71 downto 0);
      ureq_addr     : out std_logic_vector(39 downto 3);
      ureq_size     : out std_logic_vector(2 downto 0);
      ureq_srctag   : in  std_logic_vector(4 downto 0);
      ureq_mask     : out std_logic_vector(7 downto 0);
      ureq_rw_n     : out std_logic;
      ureq_data     : out std_logic_vector(63 downto 0);
      ureq_ts       : out std_logic;
      ureq_byte_req : out std_logic;
      ureq_full     : in  std_logic;
      ureq_notag    : in  std_logic;
      fresp_enable  : out std_logic;
      fresp_valid   : in  std_logic;
      fresp_ts      : in  std_logic;
      fresp_size    : in  std_logic_vector(2 downto 0);
      fresp_srctag  : in  std_logic_vector(4 downto 0);
      fresp_data    : in  std_logic_vector(63 downto 0);
      freq_enable   : out std_logic;
      freq_addr     : in  std_logic_vector(39 downto 3);
      freq_size     : in  std_logic_vector(3 downto 0);
      freq_srctag   : in  std_logic_vector(4 downto 0);
      freq_mask     : in  std_logic_vector(7 downto 0);
      freq_rw_n     : in  std_logic;
      freq_ts       : in  std_logic;
      freq_valid    : in  std_logic;
      freq_data     : in  std_logic_vector(63 downto 0);
      uresp_ts      : out std_logic;
      uresp_size    : out std_logic_vector(3 downto 0);
      uresp_srctag  : out std_logic_vector(4 downto 0);
      uresp_data    : out std_logic_vector(63 downto 0);
      uresp_full    : in  std_logic);
  end component user_app;

  -----------------------------------------------------------------------------
  -- Declare signals.
  -----------------------------------------------------------------------------

  -- Resets
  signal reset        : std_logic;
  signal reset_n      : std_logic;
  signal user_reset_n : std_logic;
  
  -- 125 MHz Rocket I/O clocks
  signal rkta_clk_in : std_logic;
  signal rktb_clk_in : std_logic;

  -- Programmable frequency clocks
  signal prog_clk_in      : std_logic;  -- Input clock after ibuf
  signal prog_clk0_dcm    : std_logic;  -- clk from dcm
  signal prog_clk0        : std_logic;  -- clk from global buffer
  signal prog_clk90       : std_logic;  -- clk w/ 90 phase from global buffer
  signal prog_dcm_lock    : std_logic;  -- dcm locked to input clock
  signal user_enable      : std_logic;
  signal k_copy_fb        : std_logic;
  signal k_copy_ddr       : std_logic;
  signal prog_clk180      : std_logic;
  signal prog_dcm_reset   : std_logic;

  -- QDR RAM to User Application Signals
  signal dw_1          : std_logic_vector(71 downto 0);  -- User write data
  signal dr_1          : std_logic_vector(71 downto 0);  -- User read data 
  signal ar_1          : std_logic_vector(19 downto 0);  -- User read address
  signal aw_1          : std_logic_vector(19 downto 0);  -- User write address
  signal r_n_1         : std_logic;                      -- User Read strobe
  signal w_n_1         : std_logic;                      -- User Write strobe
  signal bw_n_1        : std_logic_vector(7 downto 0);   -- User byte write
  signal dw_2          : std_logic_vector(71 downto 0);  -- User write data
  signal dr_2          : std_logic_vector(71 downto 0);  -- User read data 
  signal ar_2          : std_logic_vector(19 downto 0);  -- User read address
  signal aw_2          : std_logic_vector(19 downto 0);  -- User write address
  signal r_n_2         : std_logic;                      -- User Read strobe
  signal w_n_2         : std_logic;                      -- User Write strobe
  signal bw_n_2        : std_logic_vector(7 downto 0);   -- User byte write
  signal dw_3          : std_logic_vector(71 downto 0);  -- User write data
  signal dr_3          : std_logic_vector(71 downto 0);  -- User read data 
  signal ar_3          : std_logic_vector(19 downto 0);  -- User read address
  signal aw_3          : std_logic_vector(19 downto 0);  -- User write address
  signal r_n_3         : std_logic;                      -- User Read strobe
  signal w_n_3         : std_logic;                      -- User Write strobe
  signal bw_n_3        : std_logic_vector(7 downto 0);   -- User byte write
  signal dw_4          : std_logic_vector(71 downto 0);  -- User write data
  signal dr_4          : std_logic_vector(71 downto 0);  -- User read data 
  signal ar_4          : std_logic_vector(19 downto 0);  -- User read address
  signal aw_4          : std_logic_vector(19 downto 0);  -- User write address
  signal r_n_4         : std_logic;                      -- User Read strobe
  signal w_n_4         : std_logic;                      -- User Write strobe
  signal bw_n_4        : std_logic_vector(7 downto 0);   -- User byte write
  signal qdr_ram_ready : std_logic;
  
  -- RT to User Application Signals
  signal ureq_addr          : std_logic_vector(39 downto 3);
  signal ureq_size          : std_logic_vector(2 downto 0);
  signal ureq_srctag        : std_logic_vector(4 downto 0);
  signal ureq_mask          : std_logic_vector(7 downto 0);
  signal ureq_rw_n          : std_logic;
  signal ureq_data          : std_logic_vector(63 downto 0);
  signal ureq_ts            : std_logic;
  signal ureq_byte_req      : std_logic;
  signal ureq_full          : std_logic;
  signal ureq_notag         : std_logic;
  signal fresp_enable       : std_logic;
  signal fresp_valid        : std_logic;
  signal fresp_ts           : std_logic;
  signal fresp_size         : std_logic_vector(2 downto 0);
  signal fresp_srctag       : std_logic_vector(4 downto 0);
  signal fresp_data         : std_logic_vector(63 downto 0);
  signal uresp_ts           : std_logic;
  signal uresp_size         : std_logic_vector(3 downto 0);
  signal uresp_srctag       : std_logic_vector(4 downto 0);
  signal uresp_data         : std_logic_vector(63 downto 0);
  signal uresp_full         : std_logic;
  signal freq_enable        : std_logic;
  signal freq_addr          : std_logic_vector(39 downto 3);
  signal freq_size          : std_logic_vector(3 downto 0);
  signal freq_srctag        : std_logic_vector(4 downto 0);
  signal freq_mask          : std_logic_vector(7 downto 0);
  signal freq_rw_n          : std_logic;
  signal freq_ts            : std_logic;
  signal freq_valid         : std_logic;
  signal freq_data          : std_logic_vector(63 downto 0);
  signal clk_200m           : std_logic;
  signal rt_ready           : std_logic;
  
  -- RT Frequency Selector to QDR II RAM Interface Signals
  signal ram_doff_n : std_logic;
  signal sim_mode   : std_logic := '0';

  -- Debug signals
  signal data_out_buf : std_logic_vector(1 downto 0);

begin  -- architecture rtl

  -- pragma translate_off
  -- pragma synthesis_off
  sim_mode <= '1';                      -- only in simulation
  -- pragma synthesis_on
  -- pragma translate_on

  -----------------------------------------------------------------------------
  -- Global Reset
  -----------------------------------------------------------------------------

  reset_ibuf : ibuf_lvcmos2
    port map (
      i => glbl_reset_n,
      o => reset_n);

  reset <= not reset_n;

  -----------------------------------------------------------------------------
  -- Programmable frequency clock.
  -----------------------------------------------------------------------------

  prog_clk_ibuf : ibufgds_lvpecl_25
    port map (
      i  => prog_clk_p,
      ib => prog_clk_n,
      o  => prog_clk_in);

  k_copy_ibuf : ibuf_hstl_i_18
    port map (
      i => k_copy_in,
      o => k_copy_fb);

  prog_clock_gen_inst: prog_clock_gen
    port map (
      reset         => prog_dcm_reset,
      prog_clk_in   => prog_clk_in,
      k_copy_fb     => k_copy_fb,
      prog_clk0     => prog_clk0,
      prog_clk0_dcm => prog_clk0_dcm,
      prog_clk90    => prog_clk90,
      prog_clk180   => prog_clk180,
      prog_dcm_lock => prog_dcm_lock);

  k_copy_inst : fddrrse
    port map (
      q  => k_copy_ddr,
      d0 => '1',
      d1 => '0',
      c0 => prog_clk0,
      c1 => prog_clk180,
      ce => '1',
      r  => reset,
      s  => '0');

  k_copy_obuf : obuf_hstl_i_18
    port map (
      i => k_copy_ddr,
      o => k_copy_out);

  user_en_reg : process (prog_clk90, reset_n) is
  begin  -- process logic_enable
    if reset_n = '0' then               -- asynchronous reset (active low)
      user_enable <= '0';
    elsif prog_clk90'event and prog_clk90 = '1' then  -- rising clock edge
      user_enable <= prog_dcm_lock;
    end if;
  end process user_en_reg;

  -- The prog_clk0_out signal is not used. It is only here to work around a
  -- bug in the mapping process that otherwise prevents the clock macro from
  -- being used.
  prog_clk0_obuf : obuf_lvcmos25
    port map (
      i => prog_clk0_dcm,
      o => prog_clk0_out);

  
  -----------------------------------------------------------------------------
  -- Rocket I/O Clocks
  -- rkta_clk is at the top of the chip, rktb_clk is at the bottom.
  -----------------------------------------------------------------------------
  rkta_clk_ibuf : ibufgds_lvpecl_25
    port map (
      i  => rkta_clk_p,
      ib => rkta_clk_n,
      o  => rkta_clk_in);

  rktb_clk_ibuf : ibufgds_lvpecl_25
    port map (
      i  => rktb_clk_p,
      ib => rktb_clk_n,
      o  => rktb_clk_in);

  -----------------------------------------------------------------------------
  -- RapidArray Transport Interface to the Hobit FPGA
  -----------------------------------------------------------------------------

  rt_core_inst : rt_core
    port map (
      reset_n            => reset_n,
      user_clk           => prog_clk90,
      user_enable        => user_enable,
      ureq_addr          => ureq_addr,
      ureq_size          => ureq_size,
      ureq_srctag        => ureq_srctag,
      ureq_mask          => ureq_mask,
      ureq_rw_n          => ureq_rw_n,
      ureq_data          => ureq_data,
      ureq_ts            => ureq_ts,
      ureq_byte_req      => ureq_byte_req,
      ureq_full          => ureq_full,
      ureq_notag         => ureq_notag,
      fresp_enable       => fresp_enable,
      fresp_valid        => fresp_valid,
      fresp_ts           => fresp_ts,
      fresp_size         => fresp_size,
      fresp_srctag       => fresp_srctag,
      fresp_data         => fresp_data,
      uresp_ts           => uresp_ts,
      uresp_size         => uresp_size,
      uresp_srctag       => uresp_srctag,
      uresp_data         => uresp_data,
      uresp_full         => uresp_full,
      freq_enable        => freq_enable,
      freq_addr          => freq_addr,
      freq_size          => freq_size,
      freq_srctag        => freq_srctag,
      freq_mask          => freq_mask,
      freq_rw_n          => freq_rw_n,
      freq_ts            => freq_ts,
      freq_valid         => freq_valid,
      freq_data          => freq_data,
      rt_rx_clk_p        => rt_rx_clk_p,
      rt_rx_clk_n        => rt_rx_clk_n,
      rt_rx_ctl_p        => rt_rx_ctl_p,
      rt_rx_ctl_n        => rt_rx_ctl_n,
      rt_rx_cad_p        => rt_rx_cad_p,
      rt_rx_cad_n        => rt_rx_cad_n,
      rt_rx_par_p        => rt_rx_par_p,
      rt_rx_par_n        => rt_rx_par_n,
      rt_clk_p           => rt_clk_p,
      rt_clk_n           => rt_clk_n,
      rt_tx_clk_p        => rt_tx_clk_p,
      rt_tx_clk_n        => rt_tx_clk_n,
      rt_tx_ctl_p        => rt_tx_ctl_p,
      rt_tx_ctl_n        => rt_tx_ctl_n,
      rt_tx_cad_p        => rt_tx_cad_p,
      rt_tx_cad_n        => rt_tx_cad_n,
      rt_tx_par_p        => rt_tx_par_p,
      rt_tx_par_n        => rt_tx_par_n,
      clk_200m           => clk_200m,
      rt_ready           => rt_ready,
      host_par           => host_par,
      host_as            => host_as,
      host_ad            => host_ad,
      host_rw            => host_rw,
      host_cs_n          => host_cs_n,
      user_reset_n       => user_reset_n,
      host_int_n         => host_int_n,
      host_grn_led       => host_grn_led,
      ram_doff_n         => ram_doff_n,
      np_load            => np_load,
      s_clock            => s_clock,
      s_load             => s_load,
      s_data             => s_data,
      prog_en_n          => prog_en_n,
      prog_dcm_reset     => prog_dcm_reset
      );

  -----------------------------------------------------------------------------
  -- QDR II RAM Interface
  -----------------------------------------------------------------------------

  qdr2_core_inst : qdr2_core
    port map (
      sim_mode     => sim_mode,
      reset        => reset,
      qdr_clk0     => prog_clk0,
      qdr_clk90    => prog_clk90,
      locked_dcm   => prog_dcm_lock,
      ram_rdy      => qdr_ram_ready,
      dw_1         => dw_1,
      dr_1         => dr_1,
      ar_1         => ar_1,
      aw_1         => aw_1,
      r_n_1        => r_n_1,
      w_n_1        => w_n_1,
      bw_n_1       => bw_n_1,
      doff_n_1     => ram_doff_n,
      dw_2         => dw_2,
      dr_2         => dr_2,
      ar_2         => ar_2,
      aw_2         => aw_2,
      r_n_2        => r_n_2,
      w_n_2        => w_n_2,
      bw_n_2       => bw_n_2,
      doff_n_2     => ram_doff_n,
      dw_3         => dw_3,
      dr_3         => dr_3,
      ar_3         => ar_3,
      aw_3         => aw_3,
      r_n_3        => r_n_3,
      w_n_3        => w_n_3,
      bw_n_3       => bw_n_3,
      doff_n_3     => ram_doff_n,
      dw_4         => dw_4,
      dr_4         => dr_4,
      ar_4         => ar_4,
      aw_4         => aw_4,
      r_n_4        => r_n_4,
      w_n_4        => w_n_4,
      bw_n_4       => bw_n_4,
      doff_n_4     => ram_doff_n,
      qdr_rfb_n_1  => qdr_rfb_n_1,
      qdr_dr_1     => qdr_dr_1,
      qdr_dw_1     => qdr_dw_1,
      qdr_sa_1     => qdr_sa_1,
      qdr_r_n_1    => qdr_r_n_1,
      qdr_w_n_1    => qdr_w_n_1,
      qdr_bw_n_1   => qdr_bw_n_1,
      qdr_k_1      => qdr_k_1,
      qdr_k_n_1    => qdr_k_n_1,
      qdr_cq_1     => qdr_cq_1,
      qdr_rext_n_1 => qdr_rext_n_1,
      qdr_doff_n_1 => qdr_doff_n_1,
      qdr_rfb_n_2  => qdr_rfb_n_2,
      qdr_dr_2     => qdr_dr_2,
      qdr_dw_2     => qdr_dw_2,
      qdr_sa_2     => qdr_sa_2,
      qdr_r_n_2    => qdr_r_n_2,
      qdr_w_n_2    => qdr_w_n_2,
      qdr_bw_n_2   => qdr_bw_n_2,
      qdr_k_2      => qdr_k_2,
      qdr_k_n_2    => qdr_k_n_2,
      qdr_cq_2     => qdr_cq_2,
      qdr_rext_n_2 => qdr_rext_n_2,
      qdr_doff_n_2 => qdr_doff_n_2,
      qdr_rfb_n_3  => qdr_rfb_n_3,
      qdr_dr_3     => qdr_dr_3,
      qdr_dw_3     => qdr_dw_3,
      qdr_sa_3     => qdr_sa_3,
      qdr_r_n_3    => qdr_r_n_3,
      qdr_w_n_3    => qdr_w_n_3,
      qdr_bw_n_3   => qdr_bw_n_3,
      qdr_k_3      => qdr_k_3,
      qdr_k_n_3    => qdr_k_n_3,
      qdr_cq_3     => qdr_cq_3,
      qdr_rext_n_3 => qdr_rext_n_3,
      qdr_doff_n_3 => qdr_doff_n_3,
      qdr_rfb_n_4  => qdr_rfb_n_4,
      qdr_dr_4     => qdr_dr_4,
      qdr_dw_4     => qdr_dw_4,
      qdr_sa_4     => qdr_sa_4,
      qdr_r_n_4    => qdr_r_n_4,
      qdr_w_n_4    => qdr_w_n_4,
      qdr_bw_n_4   => qdr_bw_n_4,
      qdr_k_4      => qdr_k_4,
      qdr_k_n_4    => qdr_k_n_4,
      qdr_cq_4     => qdr_cq_4,
      qdr_rext_n_4 => qdr_rext_n_4,
      qdr_doff_n_4 => qdr_doff_n_4);

  -----------------------------------------------------------------------------
  -- User Application
  -----------------------------------------------------------------------------

  user_app_inst : user_app
    port map (
      reset_n       => user_reset_n,
      user_clk      => prog_clk90,
      user_enable   => user_enable,
      rt_ready      => rt_ready,
      qdr_ready     => qdr_ram_ready,
      dr_1          => dr_1,
      ar_1          => ar_1,
      aw_1          => aw_1,
      r_n_1         => r_n_1,
      w_n_1         => w_n_1,
      bw_n_1        => bw_n_1,
      dw_1          => dw_1,
      dr_2          => dr_2,
      ar_2          => ar_2,
      aw_2          => aw_2,
      r_n_2         => r_n_2,
      w_n_2         => w_n_2,
      bw_n_2        => bw_n_2,
      dw_2          => dw_2,
      dr_3          => dr_3,
      ar_3          => ar_3,
      aw_3          => aw_3,
      r_n_3         => r_n_3,
      w_n_3         => w_n_3,
      bw_n_3        => bw_n_3,
      dw_3          => dw_3,
      dr_4          => dr_4,
      ar_4          => ar_4,
      aw_4          => aw_4,
      r_n_4         => r_n_4,
      w_n_4         => w_n_4,
      bw_n_4        => bw_n_4,
      dw_4          => dw_4,
      ureq_addr     => ureq_addr,
      ureq_size     => ureq_size,
      ureq_srctag   => ureq_srctag,
      ureq_mask     => ureq_mask,
      ureq_rw_n     => ureq_rw_n,
      ureq_data     => ureq_data,
      ureq_ts       => ureq_ts,
      ureq_byte_req => ureq_byte_req,
      ureq_full     => ureq_full,
      ureq_notag    => ureq_notag,
      fresp_enable  => fresp_enable,
      fresp_valid   => fresp_valid,
      fresp_ts      => fresp_ts,
      fresp_size    => fresp_size,
      fresp_srctag  => fresp_srctag,
      fresp_data    => fresp_data,
      freq_enable   => freq_enable,
      freq_addr     => freq_addr,
      freq_size     => freq_size,
      freq_srctag   => freq_srctag,
      freq_mask     => freq_mask,
      freq_rw_n     => freq_rw_n,
      freq_ts       => freq_ts,
      freq_valid    => freq_valid,
      freq_data     => freq_data,
      uresp_ts      => uresp_ts,
      uresp_size    => uresp_size,
      uresp_srctag  => uresp_srctag,
      uresp_data    => uresp_data,
      uresp_full    => uresp_full);

  -----------------------------------------------------------------------------
  -- Debug Port
  -----------------------------------------------------------------------------

  err_obuf : obuf_hstl_i_18
    port map (
      i => '0',
      o => err);

  err_code0_obuf : obuf_hstl_i_18
    port map (
      i => '0',
      o => err_code(0));

  err_code1_obuf : obuf_hstl_i_18
    port map (
      i => '0',
      o => err_code(1));

  err_code2_obuf : obuf_hstl_i_18
    port map (
      i => '0',
      o => err_code(2));

  data_out_buf(1) <= '0';

  data_out0_obuf : obuf_hstl_i_18
    port map (
      i => data_out_buf(0),
      o => data_out(0));

  data_out_buf(0) <= '0';

  data_out1_obuf : obuf_hstl_i_18
    port map (
      i => data_out_buf(1),
      o => data_out(1));

  gnd_obuf : obuf_hstl_i_18
    port map (
      i => '0',
      o => gnd_out);

  
end architecture rtl;
