/*********************************************************************
* Copyright (c) 2005, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co_signal.c: Signal functions.
*
*********************************************************************/

#include "einlib.h"

#include <malloc.h>
#include <string.h>

#include "co.h"

// FPGA register addresses
#define DATA_OFFSET		(0*sizeof(uint64))
#define STATUS_OFFSET	(1*sizeof(uint64))

// Status bits
#define READY_MASK	0x1

extern int _co_fpga_id;

co_signal co_signal_create(const char * name)
{
	co_signal signal = malloc(sizeof(co_signal_t));	
	if ( signal != NULL ) {
		signal->name = (name != NULL) ? strdup(name) : NULL;
		signal->direction = CO_NO_DIRECTION;
		signal->io = NULL;
	}
	return signal;
}

void co_signal_attach(co_signal signal, void * io, co_stream_direction dir)
{
	if ( signal != NULL ) {
		signal->io = io;
		signal->direction = dir;
	}
}

co_error co_signal_wait(co_signal signal, int32 * vp)
{
	uint64 value;
	err_e e;
	
	if ( signal != NULL && vp != NULL ) {
		do {
			fpga_rd_appif_val(_co_fpga_id, &value, (unsigned long)signal->io + STATUS_OFFSET, &e);
		} while ( (value & READY_MASK) != 1 );
		fpga_rd_appif_val(_co_fpga_id, &value, (unsigned long)signal->io + DATA_OFFSET, &e);
		*vp = (int32)value;
	}
	return co_err_none;
}

co_error co_signal_post(co_signal signal, int32 value)
{
	err_e e;
	if ( signal != NULL ) {
		fpga_wrt_appif_val(_co_fpga_id, (uint64)value, (unsigned long)signal->io + DATA_OFFSET, 0, &e);
	}
	return co_err_none;
}

