library ieee;
use ieee.std_logic_1164.all;

library impulse;
use impulse.cray_rt_if.all;
use work.user_pkg.all;

entity rt_impulse is
  port (
    -- RT ports
    RT_reset_n : in std_logic;
    clk : in std_logic;
    RT_Req : in t_rt_req;
    RT_Resp : out t_rt_resp
  );
end rt_impulse;

architecture impl of rt_impulse is
  component FPWMi_sl is
    port (
    reset : in std_logic;
    sclk : in std_logic;
    clk : in std_logic;
    p_Producer_threshold_stream_en : in std_logic;
    p_Producer_threshold_stream_eos : in std_logic;
    p_Producer_threshold_stream_data : in std_logic_vector (31 downto 0);
    p_Producer_threshold_stream_rdy : out std_logic;
    p_Producer_pwm_stream_en : in std_logic;
    p_Producer_pwm_stream_eos : in std_logic;
    p_Producer_pwm_stream_data : in std_logic_vector (31 downto 0);
    p_Producer_pwm_stream_rdy : out std_logic;
    p_Producer_iteration_stream_en : in std_logic;
    p_Producer_iteration_stream_eos : in std_logic;
    p_Producer_iteration_stream_data : in std_logic_vector (15 downto 0);
    p_Producer_iteration_stream_rdy : out std_logic;
    p_Producer_dna_stream_en : in std_logic;
    p_Producer_dna_stream_eos : in std_logic;
    p_Producer_dna_stream_data : in std_logic_vector (7 downto 0);
    p_Producer_dna_stream_rdy : out std_logic;
    p_Consumer_start_stream_en : in std_logic;
    p_Consumer_start_stream_data : out std_logic_vector (15 downto 0);
    p_Consumer_start_stream_eos : out std_logic;
    p_Consumer_start_stream_rdy : out std_logic;
    p_Consumer_score_stream_en : in std_logic;
    p_Consumer_score_stream_data : out std_logic_vector (31 downto 0);
    p_Consumer_score_stream_eos : out std_logic;
    p_Consumer_score_stream_rdy : out std_logic
    );
  end component;

  signal p_Producer_threshold_stream_rdy : std_logic;
  signal p_Producer_threshold_stream_en : std_logic;
  signal p_Producer_threshold_stream_eos : std_logic;
  signal p_Producer_threshold_stream_idata : std_logic_vector (31 downto 0);
  signal p_Producer_pwm_stream_rdy : std_logic;
  signal p_Producer_pwm_stream_en : std_logic;
  signal p_Producer_pwm_stream_eos : std_logic;
  signal p_Producer_pwm_stream_idata : std_logic_vector (31 downto 0);
  signal p_Producer_iteration_stream_rdy : std_logic;
  signal p_Producer_iteration_stream_en : std_logic;
  signal p_Producer_iteration_stream_eos : std_logic;
  signal p_Producer_iteration_stream_idata : std_logic_vector (15 downto 0);
  signal p_Producer_dna_stream_rdy : std_logic;
  signal p_Producer_dna_stream_en : std_logic;
  signal p_Producer_dna_stream_eos : std_logic;
  signal p_Producer_dna_stream_idata : std_logic_vector (7 downto 0);
  signal p_Consumer_start_stream_rdy : std_logic;
  signal p_Consumer_start_stream_en : std_logic;
  signal p_Consumer_start_stream_eos : std_logic;
  signal p_Consumer_start_stream_idata : std_logic_vector (15 downto 0);
  signal p_Consumer_score_stream_rdy : std_logic;
  signal p_Consumer_score_stream_en : std_logic;
  signal p_Consumer_score_stream_eos : std_logic;
  signal p_Consumer_score_stream_idata : std_logic_vector (31 downto 0);

  -- rt signals
  signal rt_reset : std_logic;
  signal rt_conn_sel : std_logic_vector (2 downto 0);
  signal rt_conn_addr : std_logic_vector (1 downto 0);
  signal rt_req_wdata : std_logic_vector (63 downto 0);
  signal rt_resp_rdata : std_logic_vector (63 downto 0);
  -- end rt signals
  signal p_Producer_threshold_stream_rt_wr : std_logic;
  signal p_Producer_threshold_stream_rt_rd : std_logic;
  signal p_Producer_threshold_stream_rt_odata : std_logic_vector (63 downto 0);
  signal p_Producer_threshold_stream_rt_idata : std_logic_vector (63 downto 0);
  signal p_Producer_threshold_stream_rt_idata_reg : std_logic_vector (63 downto 0);
  signal p_Producer_threshold_stream_rt_resp_bytes_sent : std_logic_vector (3 downto 0);
  signal p_Producer_threshold_stream_rt_resp_complete : std_logic;
  signal p_Producer_pwm_stream_rt_wr : std_logic;
  signal p_Producer_pwm_stream_rt_rd : std_logic;
  signal p_Producer_pwm_stream_rt_odata : std_logic_vector (63 downto 0);
  signal p_Producer_pwm_stream_rt_idata : std_logic_vector (63 downto 0);
  signal p_Producer_pwm_stream_rt_idata_reg : std_logic_vector (63 downto 0);
  signal p_Producer_pwm_stream_rt_resp_bytes_sent : std_logic_vector (3 downto 0);
  signal p_Producer_pwm_stream_rt_resp_complete : std_logic;
  signal p_Producer_iteration_stream_rt_wr : std_logic;
  signal p_Producer_iteration_stream_rt_rd : std_logic;
  signal p_Producer_iteration_stream_rt_odata : std_logic_vector (63 downto 0);
  signal p_Producer_iteration_stream_rt_idata : std_logic_vector (63 downto 0);
  signal p_Producer_iteration_stream_rt_idata_reg : std_logic_vector (63 downto 0);
  signal p_Producer_iteration_stream_rt_resp_bytes_sent : std_logic_vector (1 downto 0);
  signal p_Producer_iteration_stream_rt_resp_complete : std_logic;
  signal p_Producer_dna_stream_rt_wr : std_logic;
  signal p_Producer_dna_stream_rt_rd : std_logic;
  signal p_Producer_dna_stream_rt_odata : std_logic_vector (63 downto 0);
  signal p_Producer_dna_stream_rt_idata : std_logic_vector (63 downto 0);
  signal p_Producer_dna_stream_rt_idata_reg : std_logic_vector (63 downto 0);
  signal p_Producer_dna_stream_rt_resp_bytes_sent : std_logic_vector (0 downto 0);
  signal p_Producer_dna_stream_rt_resp_complete : std_logic;
  signal p_Consumer_start_stream_rt_wr : std_logic;
  signal p_Consumer_start_stream_rt_rd : std_logic;
  signal p_Consumer_start_stream_rt_odata : std_logic_vector (63 downto 0);
  signal p_Consumer_start_stream_rt_idata : std_logic_vector (63 downto 0);
  signal p_Consumer_start_stream_rt_idata_reg : std_logic_vector (63 downto 0);
  signal p_Consumer_start_stream_rt_resp_bytes_sent : std_logic_vector (1 downto 0);
  signal p_Consumer_start_stream_rt_resp_complete : std_logic;
  signal p_Consumer_score_stream_rt_wr : std_logic;
  signal p_Consumer_score_stream_rt_rd : std_logic;
  signal p_Consumer_score_stream_rt_odata : std_logic_vector (63 downto 0);
  signal p_Consumer_score_stream_rt_idata : std_logic_vector (63 downto 0);
  signal p_Consumer_score_stream_rt_idata_reg : std_logic_vector (63 downto 0);
  signal p_Consumer_score_stream_rt_resp_bytes_sent : std_logic_vector (3 downto 0);
  signal p_Consumer_score_stream_rt_resp_complete : std_logic;
begin
  rt_reset <= not RT_reset_n;
  rt_conn_addr <= RT_Req.addr(t_rt_block_addr'low+1 downto t_rt_block_addr'low);
  RT_Resp.busy <= '0';

  rt_req_wdata(7 downto 0) <= std_logic_vector(RT_Req.wdata(0));
  rt_req_wdata(15 downto 8) <= std_logic_vector(RT_Req.wdata(1));
  rt_req_wdata(23 downto 16) <= std_logic_vector(RT_Req.wdata(2));
  rt_req_wdata(31 downto 24) <= std_logic_vector(RT_Req.wdata(3));
  rt_req_wdata(39 downto 32) <= std_logic_vector(RT_Req.wdata(4));
  rt_req_wdata(47 downto 40) <= std_logic_vector(RT_Req.wdata(5));
  rt_req_wdata(55 downto 48) <= std_logic_vector(RT_Req.wdata(6));
  rt_req_wdata(63 downto 56) <= std_logic_vector(RT_Req.wdata(7));

  RT_Resp.rdata(0) <= rt_resp_rdata(7 downto 0);
  RT_Resp.rdata(1) <= rt_resp_rdata(15 downto 8);
  RT_Resp.rdata(2) <= rt_resp_rdata(23 downto 16);
  RT_Resp.rdata(3) <= rt_resp_rdata(31 downto 24);
  RT_Resp.rdata(4) <= rt_resp_rdata(39 downto 32);
  RT_Resp.rdata(5) <= rt_resp_rdata(47 downto 40);
  RT_Resp.rdata(6) <= rt_resp_rdata(55 downto 48);
  RT_Resp.rdata(7) <= rt_resp_rdata(63 downto 56);
  rt_conn_sel <= RT_Req.addr(2+t_rt_block_addr'low+2 downto t_rt_block_addr'low+2);

  rt_resp_complete : process ( clk )
  begin
    if ( clk'event and clk = '1' ) then
      if ( p_Producer_threshold_stream_rt_rd = '1' ) then
        for byte in p_Producer_threshold_stream_rt_resp_bytes_sent'range loop
          p_Producer_threshold_stream_rt_resp_bytes_sent(byte) <= p_Producer_threshold_stream_rt_resp_bytes_sent(byte) or RT_Req.mask(byte);
        end loop;
        if ( p_Producer_threshold_stream_rt_resp_bytes_sent = "1111" ) then
          p_Producer_threshold_stream_rt_resp_bytes_sent <= (others => '0');
        end if;
      elsif ( p_Producer_pwm_stream_rt_rd = '1' ) then
        for byte in p_Producer_pwm_stream_rt_resp_bytes_sent'range loop
          p_Producer_pwm_stream_rt_resp_bytes_sent(byte) <= p_Producer_pwm_stream_rt_resp_bytes_sent(byte) or RT_Req.mask(byte);
        end loop;
        if ( p_Producer_pwm_stream_rt_resp_bytes_sent = "1111" ) then
          p_Producer_pwm_stream_rt_resp_bytes_sent <= (others => '0');
        end if;
      elsif ( p_Producer_iteration_stream_rt_rd = '1' ) then
        for byte in p_Producer_iteration_stream_rt_resp_bytes_sent'range loop
          p_Producer_iteration_stream_rt_resp_bytes_sent(byte) <= p_Producer_iteration_stream_rt_resp_bytes_sent(byte) or RT_Req.mask(byte);
        end loop;
        if ( p_Producer_iteration_stream_rt_resp_bytes_sent = "11" ) then
          p_Producer_iteration_stream_rt_resp_bytes_sent <= (others => '0');
        end if;
      elsif ( p_Producer_dna_stream_rt_rd = '1' ) then
        for byte in p_Producer_dna_stream_rt_resp_bytes_sent'range loop
          p_Producer_dna_stream_rt_resp_bytes_sent(byte) <= p_Producer_dna_stream_rt_resp_bytes_sent(byte) or RT_Req.mask(byte);
        end loop;
        if ( p_Producer_dna_stream_rt_resp_bytes_sent = "1" ) then
          p_Producer_dna_stream_rt_resp_bytes_sent <= (others => '0');
        end if;
      elsif ( p_Consumer_start_stream_rt_rd = '1' ) then
        for byte in p_Consumer_start_stream_rt_resp_bytes_sent'range loop
          p_Consumer_start_stream_rt_resp_bytes_sent(byte) <= p_Consumer_start_stream_rt_resp_bytes_sent(byte) or RT_Req.mask(byte);
        end loop;
        if ( p_Consumer_start_stream_rt_resp_bytes_sent = "11" ) then
          p_Consumer_start_stream_rt_resp_bytes_sent <= (others => '0');
        end if;
      elsif ( p_Consumer_score_stream_rt_rd = '1' ) then
        for byte in p_Consumer_score_stream_rt_resp_bytes_sent'range loop
          p_Consumer_score_stream_rt_resp_bytes_sent(byte) <= p_Consumer_score_stream_rt_resp_bytes_sent(byte) or RT_Req.mask(byte);
        end loop;
        if ( p_Consumer_score_stream_rt_resp_bytes_sent = "1111" ) then
          p_Consumer_score_stream_rt_resp_bytes_sent <= (others => '0');
        end if;
      end if;
    end if;
  end process;

  rt_idata_reg : process ( clk )
  begin
    if ( clk'event and clk = '1' ) then
      if ( p_Producer_threshold_stream_rt_rd = '1' and p_Producer_threshold_stream_rt_resp_complete = '1' ) then
        p_Producer_threshold_stream_rt_idata_reg <= p_Producer_threshold_stream_rt_idata;
      elsif ( p_Producer_pwm_stream_rt_rd = '1' and p_Producer_pwm_stream_rt_resp_complete = '1' ) then
        p_Producer_pwm_stream_rt_idata_reg <= p_Producer_pwm_stream_rt_idata;
      elsif ( p_Producer_iteration_stream_rt_rd = '1' and p_Producer_iteration_stream_rt_resp_complete = '1' ) then
        p_Producer_iteration_stream_rt_idata_reg <= p_Producer_iteration_stream_rt_idata;
      elsif ( p_Producer_dna_stream_rt_rd = '1' and p_Producer_dna_stream_rt_resp_complete = '1' ) then
        p_Producer_dna_stream_rt_idata_reg <= p_Producer_dna_stream_rt_idata;
      elsif ( p_Consumer_start_stream_rt_rd = '1' and p_Consumer_start_stream_rt_resp_complete = '1' ) then
        p_Consumer_start_stream_rt_idata_reg <= p_Consumer_start_stream_rt_idata;
      elsif ( p_Consumer_score_stream_rt_rd = '1' and p_Consumer_score_stream_rt_resp_complete = '1' ) then
        p_Consumer_score_stream_rt_idata_reg <= p_Consumer_score_stream_rt_idata;
      end if;
    end if;
  end process;

  rt_idata_reg_mux: process (p_Producer_threshold_stream_rt_idata_reg, p_Producer_pwm_stream_rt_idata_reg, p_Producer_iteration_stream_rt_idata_reg, p_Producer_dna_stream_rt_idata_reg, p_Consumer_start_stream_rt_idata_reg, p_Consumer_score_stream_rt_idata_reg, rt_conn_sel) is
  begin
    case rt_conn_sel is
      when "000" => rt_resp_rdata <= p_Producer_threshold_stream_rt_idata_reg;
      when "001" => rt_resp_rdata <= p_Producer_pwm_stream_rt_idata_reg;
      when "010" => rt_resp_rdata <= p_Producer_iteration_stream_rt_idata_reg;
      when "011" => rt_resp_rdata <= p_Producer_dna_stream_rt_idata_reg;
      when "100" => rt_resp_rdata <= p_Consumer_start_stream_rt_idata_reg;
      when "101" => rt_resp_rdata <= p_Consumer_score_stream_rt_idata_reg;
      when others => rt_resp_rdata <= (others => 'X');
    end case;
  end process rt_idata_reg_mux;


  p_Producer_threshold_stream_rt_wr <= std_logic(RT_Req.wr(impulse)) when rt_conn_sel = "000" else '0';
  p_Producer_threshold_stream_rt_rd <= std_logic(RT_Req.rd(impulse)) when rt_conn_sel = "000" else '0';
  p_Producer_threshold_stream_rt_odata <= rt_req_wdata;
  p_Producer_threshold_stream_rt_resp_complete <= '1' when (p_Producer_threshold_stream_rt_resp_bytes_sent = "0000") else '0';

  p_Producer_threshold_stream_if: rt_to_stream
    generic map (
      datawidth => 32
    )
    port map (
      rt_reset_n => RT_reset_n,
      clk => clk,
      rt_wr => p_Producer_threshold_stream_rt_wr,
      rt_rd => p_Producer_threshold_stream_rt_rd,
      rt_wdata => p_Producer_threshold_stream_rt_odata,
      rt_rdata => p_Producer_threshold_stream_rt_idata,
      rt_addr => rt_conn_addr,
      rt_resp_complete => p_Producer_threshold_stream_rt_resp_complete,
      stream_en => p_Producer_threshold_stream_en,
      stream_rdy => p_Producer_threshold_stream_rdy,
      stream_eos => p_Producer_threshold_stream_eos,
      stream_data => p_Producer_threshold_stream_idata
    );

  p_Producer_pwm_stream_rt_wr <= std_logic(RT_Req.wr(impulse)) when rt_conn_sel = "001" else '0';
  p_Producer_pwm_stream_rt_rd <= std_logic(RT_Req.rd(impulse)) when rt_conn_sel = "001" else '0';
  p_Producer_pwm_stream_rt_odata <= rt_req_wdata;
  p_Producer_pwm_stream_rt_resp_complete <= '1' when (p_Producer_pwm_stream_rt_resp_bytes_sent = "0000") else '0';

  p_Producer_pwm_stream_if: rt_to_stream
    generic map (
      datawidth => 32
    )
    port map (
      rt_reset_n => RT_reset_n,
      clk => clk,
      rt_wr => p_Producer_pwm_stream_rt_wr,
      rt_rd => p_Producer_pwm_stream_rt_rd,
      rt_wdata => p_Producer_pwm_stream_rt_odata,
      rt_rdata => p_Producer_pwm_stream_rt_idata,
      rt_addr => rt_conn_addr,
      rt_resp_complete => p_Producer_pwm_stream_rt_resp_complete,
      stream_en => p_Producer_pwm_stream_en,
      stream_rdy => p_Producer_pwm_stream_rdy,
      stream_eos => p_Producer_pwm_stream_eos,
      stream_data => p_Producer_pwm_stream_idata
    );

  p_Producer_iteration_stream_rt_wr <= std_logic(RT_Req.wr(impulse)) when rt_conn_sel = "010" else '0';
  p_Producer_iteration_stream_rt_rd <= std_logic(RT_Req.rd(impulse)) when rt_conn_sel = "010" else '0';
  p_Producer_iteration_stream_rt_odata <= rt_req_wdata;
  p_Producer_iteration_stream_rt_resp_complete <= '1' when (p_Producer_iteration_stream_rt_resp_bytes_sent = "00") else '0';

  p_Producer_iteration_stream_if: rt_to_stream
    generic map (
      datawidth => 16
    )
    port map (
      rt_reset_n => RT_reset_n,
      clk => clk,
      rt_wr => p_Producer_iteration_stream_rt_wr,
      rt_rd => p_Producer_iteration_stream_rt_rd,
      rt_wdata => p_Producer_iteration_stream_rt_odata,
      rt_rdata => p_Producer_iteration_stream_rt_idata,
      rt_addr => rt_conn_addr,
      rt_resp_complete => p_Producer_iteration_stream_rt_resp_complete,
      stream_en => p_Producer_iteration_stream_en,
      stream_rdy => p_Producer_iteration_stream_rdy,
      stream_eos => p_Producer_iteration_stream_eos,
      stream_data => p_Producer_iteration_stream_idata
    );

  p_Producer_dna_stream_rt_wr <= std_logic(RT_Req.wr(impulse)) when rt_conn_sel = "011" else '0';
  p_Producer_dna_stream_rt_rd <= std_logic(RT_Req.rd(impulse)) when rt_conn_sel = "011" else '0';
  p_Producer_dna_stream_rt_odata <= rt_req_wdata;
  p_Producer_dna_stream_rt_resp_complete <= '1' when (p_Producer_dna_stream_rt_resp_bytes_sent = "0") else '0';

  p_Producer_dna_stream_if: rt_to_stream
    generic map (
      datawidth => 8
    )
    port map (
      rt_reset_n => RT_reset_n,
      clk => clk,
      rt_wr => p_Producer_dna_stream_rt_wr,
      rt_rd => p_Producer_dna_stream_rt_rd,
      rt_wdata => p_Producer_dna_stream_rt_odata,
      rt_rdata => p_Producer_dna_stream_rt_idata,
      rt_addr => rt_conn_addr,
      rt_resp_complete => p_Producer_dna_stream_rt_resp_complete,
      stream_en => p_Producer_dna_stream_en,
      stream_rdy => p_Producer_dna_stream_rdy,
      stream_eos => p_Producer_dna_stream_eos,
      stream_data => p_Producer_dna_stream_idata
    );

  p_Consumer_start_stream_rt_wr <= '0';
  p_Consumer_start_stream_rt_rd <= std_logic(RT_Req.rd(impulse)) when rt_conn_sel = "100" else '0';
  p_Consumer_start_stream_rt_odata <= (others => 'X');
  p_Consumer_start_stream_rt_resp_complete <= '1' when (p_Consumer_start_stream_rt_resp_bytes_sent = "00") else '0';

  p_Consumer_start_stream_if: stream_to_rt
    generic map (
      datawidth => 16
    )
    port map (
      rt_reset_n => RT_reset_n,
      clk => clk,
      rt_wr => p_Consumer_start_stream_rt_wr,
      rt_rd => p_Consumer_start_stream_rt_rd,
      rt_wdata => p_Consumer_start_stream_rt_odata,
      rt_rdata => p_Consumer_start_stream_rt_idata,
      rt_addr => rt_conn_addr,
      rt_resp_complete => p_Consumer_start_stream_rt_resp_complete,
      stream_en => p_Consumer_start_stream_en,
      stream_rdy => p_Consumer_start_stream_rdy,
      stream_eos => p_Consumer_start_stream_eos,
      stream_data => p_Consumer_start_stream_idata
    );

  p_Consumer_score_stream_rt_wr <= '0';
  p_Consumer_score_stream_rt_rd <= std_logic(RT_Req.rd(impulse)) when rt_conn_sel = "101" else '0';
  p_Consumer_score_stream_rt_odata <= (others => 'X');
  p_Consumer_score_stream_rt_resp_complete <= '1' when (p_Consumer_score_stream_rt_resp_bytes_sent = "0000") else '0';

  p_Consumer_score_stream_if: stream_to_rt
    generic map (
      datawidth => 32
    )
    port map (
      rt_reset_n => RT_reset_n,
      clk => clk,
      rt_wr => p_Consumer_score_stream_rt_wr,
      rt_rd => p_Consumer_score_stream_rt_rd,
      rt_wdata => p_Consumer_score_stream_rt_odata,
      rt_rdata => p_Consumer_score_stream_rt_idata,
      rt_addr => rt_conn_addr,
      rt_resp_complete => p_Consumer_score_stream_rt_resp_complete,
      stream_en => p_Consumer_score_stream_en,
      stream_rdy => p_Consumer_score_stream_rdy,
      stream_eos => p_Consumer_score_stream_eos,
      stream_data => p_Consumer_score_stream_idata
    );

  FPWMi_sl_0: FPWMi_sl
    port map (
      rt_reset,
      clk,
      clk,
      p_Producer_threshold_stream_en,
      p_Producer_threshold_stream_eos,
      p_Producer_threshold_stream_idata,
      p_Producer_threshold_stream_rdy,
      p_Producer_pwm_stream_en,
      p_Producer_pwm_stream_eos,
      p_Producer_pwm_stream_idata,
      p_Producer_pwm_stream_rdy,
      p_Producer_iteration_stream_en,
      p_Producer_iteration_stream_eos,
      p_Producer_iteration_stream_idata,
      p_Producer_iteration_stream_rdy,
      p_Producer_dna_stream_en,
      p_Producer_dna_stream_eos,
      p_Producer_dna_stream_idata,
      p_Producer_dna_stream_rdy,
      p_Consumer_start_stream_en,
      p_Consumer_start_stream_idata,
      p_Consumer_start_stream_eos,
      p_Consumer_start_stream_rdy,
      p_Consumer_score_stream_en,
      p_Consumer_score_stream_idata,
      p_Consumer_score_stream_eos,
      p_Consumer_score_stream_rdy);

end;

