/*********************************************************************
* Copyright (c) 2004-2005, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co.h: Impulse C interface for the Cray XD1 platform.
*
* Definitions of Impulse C functions, including:
*   - process creation and execution
*   - bit operations
*   - streams
*   - signals
*   - registers
*   - shared memory
*
*********************************************************************/

#ifndef CO_H
#define CO_H

#include <stddef.h>

#define IMPULSE_C_TARGET CRAY_XD1_OPTERON
#define IMPULSE_C_BUS RT
#define REGISTER_WIDTH 64

extern int _co_fpga_id;  /* See co_init.c */

#include "co_if_sim.h"
#include "co_types.h"

extern co_process co_process_create(const char * name, co_function run, int argc, ...);
extern void co_execute(co_architecture arch);
#define cosim_architecture_config(arch, attribute, value)

/* The datatype used in stream operations is the register width of the
 * architecture, so 'width' is effectively ignored. */
extern co_type co_type_create(co_sort sort, unsigned int width);
#define INT_TYPE(width)		co_type_create(co_int_sort,width)
#define UINT_TYPE(width) 	co_type_create(co_uint_sort,width)
#define CHAR_TYPE			co_type_create(co_int_sort,8)
#define FLOAT_TYPE			co_type_create(co_float_sort,32)
#define DOUBLE_TYPE			co_type_create(co_float_sort,64)

/* Bit operations
 *
 * For consistency with desktop simulation, these are 32-bit operations. */
#define co_bit_extract(value, start_bit, num_bits) (int32)(((value) >> (start_bit)) & (~(0xffffffffU << (num_bits))))
#define co_bit_extract_u(value, start_bit, num_bits) (uint32)((start_bit) >= 32 ? 0 : ((value) >> (start_bit)) & (~(0xffffffffU << (num_bits))))
#define co_bit_insert(destination, dest_start_bit, num_bits, source) \
	(int32)( \
	    ( (destination) & ( 0xffffffffU << ( (dest_start_bit) + (num_bits) ) ) ) | \
        ( ( (source) << (dest_start_bit) ) & ~( ( 0xffffffffU << ( (dest_start_bit) + (num_bits) ) ) | ~( 0xffffffffU << (dest_start_bit) ) ) ) | \
        ( (destination) & ~( 0xffffffffU << (dest_start_bit) ) ) \
	)
#define co_bit_insert_u(destination, dest_start_bit, num_bits, source) (uint32)co_bit_insert((int32)(destination), (dest_start_bit), (num_bits), (int32)(source))

/* Streams
 *
 * 'bufferdepth' and 'datatype' are ignored when creating a stream.  The buffer 
 * depth is specified in hardware and the datatype is limited to the register
 * width. */
extern co_stream co_stream_create(const char * name, co_type datatype, unsigned int bufferdepth);
extern void co_stream_attach(co_stream stream, void * io, co_stream_direction dir);
extern co_error co_stream_write(co_stream stream, const void * buffer, unsigned int buffersize);
extern co_error co_stream_read(co_stream stream, void * buffer, unsigned int buffersize);
extern int co_stream_read_nb(co_stream stream, void * buffer, unsigned int buffersize);
extern int co_stream_eos(co_stream stream);
extern co_error co_stream_close(co_stream stream);
extern co_error co_stream_free(co_stream stream);

#define co_stream_open(stream, mode, type)

/* Signals */
extern co_signal co_signal_create(const char * name);
extern void co_signal_attach(co_signal signal, void * io, co_stream_direction dir);
extern co_error co_signal_wait(co_signal signal, int32 * vp);
extern co_error co_signal_post(co_signal signal, int32 value);

#define co_par_break()

#include "co_math.h"

#endif  /* CO_H */

